/* 3-WAY ALIGNMENT USING CARRILLO-LIPMAN strategy

Copyright (c) 2024 Mahbubeh Askari-Rad
All Rights Reserved.

This program takes a set of sequences and align all possible triplets with 
3 dimensional dynamic programming simplifies by Carrillo-Lipman method. 
The output is distance matrix of input sequences, which is the average of 
evolutionary distance over all triplets containing sequence pair.
The input coudl be sequences including gaps (for-example the generated 
protein sequences generated by IQTREE) 
The input file should be a fasta file.

This code calculate the three Carrillo-Lipman bounds and reduce the searching area
of dynamic progrmming to path satisfying three bounds.

User is asked to enter GO, GE, num_seqs which are the values for 
gap opening, gap extension, number of sequences.
GO and GE could be any positive integers.

The program written in C and runs under  Mac system. 
The gapless sequences are stored in a fasta file. The input file should use
upper case alphabet for amino acids.

The name of input fasta file should be data.txt
we run the code with && gcc Main_file.c Readsequences.c -o main_file.c
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <stdbool.h>
#include "Readsequences.h"

int GO;
int GE;
int num_seqs;

char AA[] = {'A',  'R',  'N',  'D',  'C',  'Q',  'E', 'G',  'H',  'I',  'L',  'K',  'M', 'F',  'P', 'S', 'T',  'W',  'Y', 'V', '\0'};
int  Amino_num[25] = {1,0,2,3,4,5,6,7,8,0,9,10,11,12,0,13,14,15,16,17,0,18,19,0,20};

int seq1_length; // no. of cols
int seq2_length; // no. of rows
int seq3_length; // no. of rows
int max_size = 1000;

int *seq_lengths;
int **AlignedTriplet;
int **AlignedTriplet2;
int alignment_length;
int e_length = 0;

int lookup_table[10000000][10];
int lookup_length;

int** Seqs;
int** reverse_Seqs;
int* ROSE_length;
int mmflag = 0;
double fracchange = 1;
int num_cell_candidates;
int score12, score13, score23, score_e;

int ctr1 = 0;
int p_ct1 = 0;
int p_ct2 = 0;

int*** M;
int*** XY;
int*** XZ;
int*** YZ;
int*** X;
int*** Y;
int*** Z;


int** AlignedPair;
int** Alignment12;
int** Alignment13;
int** Alignment23;
int** lambda_e;
int** Alignment1;
int** Alignment2;
int Alignment_length12;
int Alignment_length13;
int Alignment_length23;

double  q, p, dp, d2p, lnL, slope, curve;
int iteration;
double delta;
double D;
bool neginfinity, overlap, inf;
double** d_Matrix;

int** M2;
int** MM;
int** X2;
int** Y2;
int** B0;
int** B1;
int** B2;
int** B;

int** fMM;
int** fX2;
int** fY2;

int** score_matrix_12;
int** score_matrix_13;
int** score_matrix_23;


int ct1 = 0;
int ct2 = 0;
char gaplessidx[20] ;
char data_idx[20];
char mat_idx[20];
int L12, L13, L23;

void PairDistance(int a1, int a2);
int mmMatrix(int nn1, int nn2);
int B0_func(int *s1, int *s2, int C, int R);
int B1_func(int *s1, int *s2, int C, int R);
int B2_func(int *s1, int *s2, int C, int R);
int BM_func(int* s1, int* s2, int* s3, int cell_idx);
int BXY_func(int* s1, int* s2, int* s3, int cell_idx);
int BXZ_func(int* s1, int* s2, int* s3, int cell_idx);
int BYZ_func(int* s1, int* s2, int* s3, int cell_idx);
int BX_func(int* s1, int* s2, int* s3, int cell_idx);
int BY_func(int* s1, int* s2, int* s3, int cell_idx);
int BZ_func(int* s1, int* s2, int* s3, int cell_idx);
int P_B0_func(int **s1, int *s2, int C, int R);
int P_B1_func(int **s1, int *s2, int C, int R);
int P_B2_func(int **s1, int *s2, int C, int R);

int max_function2(int num1, int num2){
    if (num1 >= num2)
        return num1;
    else
        return num2;
}

int B0_func(int *s1, int *s2, int C, int R){
    if(R == 0 && C == 0 || R < 0 || C < 0)
        return 0;

    AlignedPair[0][ct1] = s1[C-1];
    AlignedPair[1][ct2] = s2[R-1];
    ct1++;
    ct2++;
    alignment_length++;
    
    if (B0[R][C] == 0)
        B0_func(s1, s2, C-1, R-1);
    else if(B0[R][C] == 1 || B0[R][C] == 3)
        B1_func(s1, s2, C-1, R-1);
    else if(B0[R][C] == 2)
        B2_func(s1, s2, C-1, R-1);
    return 0;
}

int B1_func(int *s1, int *s2, int C, int R){

    if(R == 0 && C == 0 || R < 0 || C < 0)
        return 0;

    AlignedPair[0][ct1] = 0;
    AlignedPair[1][ct2] = s2[R-1];
    ct1++;
    ct2++;
    alignment_length++;

    if (B1[R][C] == 0)
        B0_func(s1, s2, C, R-1); 
    else if(B1[R][C] == 1)
        B1_func(s1, s2, C, R-1);
    else    
        B2_func(s1, s2, C, R-1);
    return 0;
}

int B2_func(int *s1, int *s2, int C, int R){

    if(R == 0 && C == 0 || R < 0 || C < 0)
        return 0;

    AlignedPair[0][ct1] = s1[C-1];
    AlignedPair[1][ct2] = 0;
    ct1++;
    ct2++;
    alignment_length++;

    if (B2[R][C] == 0)
        B0_func(s1, s2, C-1, R); 
    else if(B2[R][C] == 1)
        B1_func(s1, s2, C-1, R);
    else    
        B2_func(s1, s2, C-1, R);
    return 0;
}

void Alignment_2D(int* s1, int* s2, int seq1_l, int seq2_l){

    MM = malloc(sizeof(int*) * (seq2_l+1));
    X2 = malloc(sizeof(int*) * (seq2_l+1));
    Y2 = malloc(sizeof(int*) * (seq2_l+1));
    B0 = malloc(sizeof(int*) * (seq2_l+1));
    B1 = malloc(sizeof(int*) * (seq2_l+1));
    B2 = malloc(sizeof(int*) * (seq2_l+1));
    B = malloc(sizeof(int*) * (seq2_l+1));
    
    for (int i = 0; i < seq2_l+1; i++){

        MM[i] = malloc(sizeof(int) * (seq1_l + 1));
        X2[i] = malloc(sizeof(int) * (seq1_l + 1));
        Y2[i] = malloc(sizeof(int) * (seq1_l + 1));
        B0[i] = malloc(sizeof(int) * (seq1_l + 1));
        B1[i] = malloc(sizeof(int) * (seq1_l + 1));
        B2[i] = malloc(sizeof(int) * (seq1_l + 1));
        B[i] = malloc(sizeof(int) * (seq1_l + 1));
    
    }

    for (int row = 1; row < seq2_l + 1; row++){
        X2[row][0] = -GO + (-GE * (row - 1));
        Y2[row][0] = -1000000;
        MM[row][0] = -1000000;
        B0[row][0] = 1;
        B1[row][0] = 1;
        B2[row][0] = 1;
        B[row][0] = 1;
    }

    for (int col = 1; col < seq1_l + 1; col++){

        X2[0][col] = -1000000;
        Y2[0][col] = -GO + (-GE * (col - 1));
        MM[0][col] = -1000000;
        B0[0][col] = 2;
        B1[0][col] = 2;
        B2[0][col] = 2;
        B[0][col] = 2;
    }

    MM[0][0] = 0;
    X2[0][0] = 0;
    Y2[0][0] = 0;

    int s;
    for(int row = 1; row < seq2_l + 1; row++){
        for(int col = 1; col < seq1_l + 1; col++){
            
            s = mmMatrix(s1[col-1],s2[row-1]);
          
            MM[row][col] = max_function2(max_function2(MM[row-1][col-1] + s, X2[row-1][col-1] + s), Y2[row-1][col-1] + s);
            X2[row][col] = max_function2(max_function2(MM[row-1][col] - GO, X2[row-1][col] - GE), Y2[row-1][col]-GO);
            Y2[row][col] = max_function2(max_function2(MM[row][col-1] - GO, Y2[row][col-1] - GE), X2[row][col-1]-GO);

            if (MM[row][col] == MM[row-1][col-1]+s)
                B0[row][col] = 0;
            else if (MM[row][col] == X2[row-1][col-1]+s)
                B0[row][col] = 1;
            else 
                B0[row][col] = 2;
            
            if(X2[row][col] == MM[row-1][col]-GO)
                B1[row][col] = 0;
            else if(X2[row][col] == X2[row-1][col]-GE)
                B1[row][col] = 1;
            else if(X2[row][col] == Y2[row-1][col]-GO)
                B1[row][col] = 2;

            if(Y2[row][col] == MM[row][col-1]-GO)
                B2[row][col] = 0;
            else if(Y2[row][col] == X2[row][col-1]-GO)
                B2[row][col] = 1;
            else if(Y2[row][col] == Y2[row][col-1]-GE)
                B2[row][col] = 2;

        }
    }
    
    int R = seq2_l;
    int C = seq1_l;

    int max = max_function2(max_function2(MM[R][C], X2[R][C]), Y2[R][C]);

    if (max == MM[R][C])
        B0_func(s1, s2, C, R);
    else if(max == X2[R][C])
        B1_func(s1, s2, C, R);
    else if(max == Y2[R][C])
        B2_func(s1, s2, C, R);
}

void Distance_matrix(int *seq1, int *seq2, int length, int seq1_idx, int seq2_idx){
    int aa1, aa2;
    D = 0.1;
    delta = 0.05;
    iteration = 0;

    do {
        lnL = 0.0;
        slope = 0.0;
        curve = 0.0;
        neginfinity = false;
        overlap = false;
        inf = false;
        int n1, n2;

        for(int c = 0; c < length; c++){

            aa1 = seq1[c];
            aa2 = seq2[c];
            if (aa1 != 0 && aa2 != 0){
                overlap = true;

                for (int i=0; i<25; i++){
                    if (aa1 == Amino_num[i])
                        n1 = i+1;
                    if (aa2 == Amino_num[i])
                        n2 = i+1;
                }
                char c1 = (char)(n1+64);
                char c2 = (char)(n2+64);
                
                for (int i=0; i < 20; i++){
                    if (AA[i] == c1){
                        for(int j=0; j<20; j++){
                            if(AA[j] == c2){

                                q = 0.0;
                                p = 0.0;
                                dp = 0.0;
                                d2p = 0.0;
                                PairDistance(i, j); 
                                break;
                            }
                        }
                    }
                }

                if(p<=0){ // no log
                    neginfinity = true;
                   
                }else{
                    
                    lnL += log(p);
                    slope += dp/p; //(logp)'
                    curve += (d2p*p - dp*dp)/(p*p); //(logp)''
                }
            }
        }
        iteration++;
            
        if(!overlap){ // overlap = false
            //printf("\nWARNING: NO OVERLAP BETWEEN SEQUENCES %d AND %d; -1.0 WAS WRITTEN\n", seq1_idx, seq2_idx);
            d_Matrix[seq1_idx][seq2_idx] = d_Matrix[seq1_idx][seq2_idx] - 0.1;// fracchange
            iteration = 20;

        }else if (!neginfinity){ // neginfinity = false
            
            if (curve < 0){
                delta = 0.01;
                if(slope>0)
                    D += delta;
                else if (slope == 0)

                    D = D;
                else
                    D-= delta;
                
                if (D > 10000.0) {
                    //printf("\nWARNING: INFINITE DISTANCE BETWEEN SPECIES %d AND %d; -1.0 WAS WRITTEN\n", seq1_idx, seq2_idx);
                    d_Matrix[seq1_idx][seq2_idx] = d_Matrix[seq1_idx][seq2_idx] - 1.0;
                    inf = true;
                    iteration = 20;
                }

            }else{ //curve>=0
                if ((slope > 0.0 && delta < 0.0) || (slope < 0.0 && delta > 0.0))
                    delta /= -2;
                D += delta;
            }
        }
        else{  
            delta /= -2;
            D += delta;
        }
        if(D < 0.00001)
            D=0.00001;
        
    }while(iteration != 600);

    d_Matrix[seq1_idx][seq2_idx] = d_Matrix[seq1_idx][seq2_idx] + fracchange * D;
    d_Matrix[seq2_idx][seq1_idx] = d_Matrix[seq2_idx][seq1_idx] + fracchange * D;

}

int BM_func(int* s1, int* s2, int* s3, int cell_idx){

    int x = lookup_table[cell_idx][0];
    int y = lookup_table[cell_idx][1];
    int z = lookup_table[cell_idx][2];

    if(x == 0 && y == 0 && z == 0 || x < 0 || y < 0 || z < 0)
        return 0;

    int new_cell_idx;
    for(int b = cell_idx; b>=0; b--){
        if(lookup_table[b][0] == x-1 && lookup_table[b][1] == y-1 && lookup_table[b][2] == z-1){
            new_cell_idx = b;
            break;
        }
    }

    AlignedTriplet[0][ctr1] = s1[x-1];
    AlignedTriplet[1][ctr1] = s2[y-1];
    AlignedTriplet[2][ctr1] = s3[z-1];
    ctr1++;
    
    switch (lookup_table[cell_idx][3])
    {
    case 0 :
       
        BM_func(s1, s2, s3, new_cell_idx);
        break;
    case 1 :
        BXY_func(s1, s2, s3, new_cell_idx);
        break;
    case 2 :
        BXZ_func(s1, s2, s3, new_cell_idx);
        break;
    case 3 :
        BYZ_func(s1, s2, s3, new_cell_idx);
        break;
    case 4 :
        BX_func(s1, s2, s3, new_cell_idx);
        break;
    case 5 :
        BY_func(s1, s2, s3, new_cell_idx);
        break;
    case 6 :
        BZ_func(s1, s2, s3, new_cell_idx);
        break;
    default:
        break;
    }
    return 0;
}

int BXY_func(int* s1, int* s2, int* s3, int cell_idx){

    int x = lookup_table[cell_idx][0];
    int y = lookup_table[cell_idx][1];
    int z = lookup_table[cell_idx][2];

    if(x == 0 && y == 0 && z == 0 || x < 0 || y < 0 || z < 0)
        return 0;

    int new_cell_idx;
    for(int b = cell_idx; b>=0; b--){
        if(lookup_table[b][0] == x-1 && lookup_table[b][1] == y-1 && lookup_table[b][2] == z){
            new_cell_idx = b;
            break;
        }
    }
    AlignedTriplet[0][ctr1] = s1[x-1];
    AlignedTriplet[1][ctr1] = s2[y-1];
    AlignedTriplet[2][ctr1] = 0;
    ctr1++;
   
    switch (lookup_table[cell_idx][4])
    {
    case 0 :
       
        BM_func(s1, s2, s3, new_cell_idx);
        break;
    case 1 :
        BXY_func(s1, s2, s3, new_cell_idx);
        break;
    case 2 :
        BXZ_func(s1, s2, s3, new_cell_idx);
        break;
    case 3 :
        BYZ_func(s1, s2, s3, new_cell_idx);
        break;
    case 4 :
        BX_func(s1, s2, s3, new_cell_idx);
        break;
    case 5 :
        BY_func(s1, s2, s3, new_cell_idx);
        break;
    case 6 :
        BZ_func(s1, s2, s3, new_cell_idx);
        break;
    default:
        break;
    }
    return 0;
}

int BXZ_func(int* s1, int* s2, int* s3, int cell_idx){

    int x = lookup_table[cell_idx][0];
    int y = lookup_table[cell_idx][1];
    int z = lookup_table[cell_idx][2];

    if(x == 0 && y == 0 && z == 0 || x < 0 || y < 0 || z < 0)
        return 0;

    int new_cell_idx;
    for(int b = cell_idx; b>=0; b--){
        if(lookup_table[b][0] == x-1 && lookup_table[b][1] == y && lookup_table[b][2] == z-1){
            new_cell_idx = b;
            break;
        }
    }

    AlignedTriplet[0][ctr1] = s1[x-1];
    AlignedTriplet[1][ctr1] = 0;
    AlignedTriplet[2][ctr1] = s3[z-1];
    ctr1++;

    switch (lookup_table[cell_idx][5])
    {
    case 0 : 
        BM_func(s1, s2, s3, new_cell_idx);
        break;
    case 1 :
        BXY_func(s1, s2, s3, new_cell_idx);
        break;
    case 2 :
        BXZ_func(s1, s2, s3, new_cell_idx);
        break;
    case 3 :
        BYZ_func(s1, s2, s3, new_cell_idx);
        break;
    case 4 :
        BX_func(s1, s2, s3, new_cell_idx);
        break;
    case 5 :
        BY_func(s1, s2, s3, new_cell_idx);
        break;
    case 6 :
        BZ_func(s1, s2, s3, new_cell_idx);
        break;
    default:
        break;
    }
    return 0;
}

int BYZ_func(int* s1, int* s2, int* s3, int cell_idx){

    int x = lookup_table[cell_idx][0];
    int y = lookup_table[cell_idx][1];
    int z = lookup_table[cell_idx][2];

    if(x == 0 && y == 0 && z == 0 || x < 0 || y < 0 || z < 0)
        return 0;

    int new_cell_idx;
    for(int b = cell_idx; b>=0; b--){
        if(lookup_table[b][0] == x && lookup_table[b][1] == y-1 && lookup_table[b][2] == z-1){
            new_cell_idx = b;
            break;
        }
    }

    AlignedTriplet[0][ctr1] = 0;
    AlignedTriplet[1][ctr1] = s2[y-1];
    AlignedTriplet[2][ctr1] = s3[z-1];
    ctr1++;

    switch (lookup_table[cell_idx][6])
    {
    case 0 :
       
        BM_func(s1, s2, s3, new_cell_idx);
        break;
    case 1 :
        BXY_func(s1, s2, s3, new_cell_idx);
        break;
    case 2 :
        BXZ_func(s1, s2, s3, new_cell_idx);
        break;
    case 3 :
        BYZ_func(s1, s2, s3, new_cell_idx);
        break;
    case 4 :
        BX_func(s1, s2, s3, new_cell_idx);
        break;
    case 5 :
        BY_func(s1, s2, s3, new_cell_idx);
        break;
    case 6 :
        BZ_func(s1, s2, s3, new_cell_idx);
        break;
    default:
        break;
    }
    return 0;
}

int BX_func(int* s1, int* s2, int* s3, int cell_idx){

    int x = lookup_table[cell_idx][0];
    int y = lookup_table[cell_idx][1];
    int z = lookup_table[cell_idx][2];

    if(x == 0 && y == 0 && z == 0 || x < 0 || y < 0 || z < 0)
        return 0;

    int new_cell_idx;
    for(int b = cell_idx; b>=0; b--){
        if(lookup_table[b][0] == x-1 && lookup_table[b][1] == y && lookup_table[b][2] == z){
            new_cell_idx = b;
            break;
        }
    }

    AlignedTriplet[0][ctr1] = s1[x-1];
    AlignedTriplet[1][ctr1] = 0;
    AlignedTriplet[2][ctr1] = 0;
    ctr1++;
    
    switch (lookup_table[cell_idx][7])
    {
    case 0 :
        BM_func(s1, s2, s3, new_cell_idx);
        break;
    case 1 :
        BXY_func(s1, s2, s3, new_cell_idx);
        break;
    case 2 :
        BXZ_func(s1, s2, s3, new_cell_idx);
        break;
    case 3 :
        BYZ_func(s1, s2, s3, new_cell_idx);
        break;
    case 4 :
        BX_func(s1, s2, s3, new_cell_idx);
        break;
    case 5 :
        BY_func(s1, s2, s3, new_cell_idx);
        break;
    case 6 :
        BZ_func(s1, s2, s3, new_cell_idx);
        break;
    default:
        break;
    }
    return 0;
}

int BY_func(int* s1, int* s2, int* s3, int cell_idx){

    int x = lookup_table[cell_idx][0];
    int y = lookup_table[cell_idx][1];
    int z = lookup_table[cell_idx][2];

    if(x == 0 && y == 0 && z == 0 || x < 0 || y < 0 || z < 0)
        return 0;

    AlignedTriplet[0][ctr1] = 0;
    AlignedTriplet[1][ctr1] = s2[y-1];
    AlignedTriplet[2][ctr1] = 0;
    ctr1++;
    
    int new_cell_idx;
    for(int b = cell_idx; b>=0; b--){
        if(lookup_table[b][0] == x && lookup_table[b][1] == y-1 && lookup_table[b][2] == z){
            new_cell_idx = b;
            break;
        }
    }

    switch (lookup_table[cell_idx][8])
    {
    case 0 :
        BM_func(s1, s2, s3, new_cell_idx);
        break;
    case 1 :
        BXY_func(s1, s2, s3, new_cell_idx);
        break;
    case 2 :
        BXZ_func(s1, s2, s3, new_cell_idx);
        break;
    case 3 :
        BYZ_func(s1, s2, s3, new_cell_idx);
        break;
    case 4 :
        BX_func(s1, s2, s3, new_cell_idx);
        break;
    case 5 :
        BY_func(s1, s2, s3, new_cell_idx);
        break;
    case 6 :
        BZ_func(s1, s2, s3, new_cell_idx);
        break;
    default:
        break;
    }
    
    return 0;
}

int BZ_func(int* s1, int* s2, int* s3, int cell_idx){

    int x = lookup_table[cell_idx][0];
    int y = lookup_table[cell_idx][1];
    int z = lookup_table[cell_idx][2];

    if(x == 0 && y == 0 && z == 0 || x < 0 || y < 0 || z < 0)
        return 0;

    AlignedTriplet[0][ctr1] = 0;
    AlignedTriplet[1][ctr1] = 0;
    AlignedTriplet[2][ctr1] = s3[z-1];
    ctr1++;

    int new_cell_idx;
    for(int b = cell_idx; b>=0; b--){
        if(lookup_table[b][0] == x && lookup_table[b][1] == y && lookup_table[b][2] == z-1){
            new_cell_idx = b;
            break;
        }
    }
    switch (lookup_table[cell_idx][9])
    {
    case 0 :
        BM_func(s1, s2, s3, new_cell_idx);
        break;
    case 1 :
        BXY_func(s1, s2, s3, new_cell_idx);
        break;
    case 2 :
        BXZ_func(s1, s2, s3, new_cell_idx);
        break;
    case 3 :
        BYZ_func(s1, s2, s3, new_cell_idx);
        break;
    case 4 :
        BX_func(s1, s2, s3, new_cell_idx);
        break;
    case 5 :
        BY_func(s1, s2, s3, new_cell_idx);
        break;
    case 6 :
        BZ_func(s1, s2, s3, new_cell_idx);
        break;
    default:
        break;
    }
    return 0;
}

void dynamic_prog(int* s1, int* s2, int* s3){

    int x,y,z;

    M[0][0][1] = 0;
    XY[0][0][1] = 0;
    XZ[0][0][1] = 0;
    YZ[0][0][1] = 0;
    X[0][0][1] = 0;
    Y[0][0][1] = 0;
    Z[0][0][1] = 0;

    int values[7];
    int score = 0;
    int z_idx = 0;
    int  c = 0;
    int max, idx;

    int score12, score13, score23;

    for(int i=0; i<seq1_length+1; i++){
        for(int j=0; j<seq2_length+1; j++){

            M[i][j][0] = -1000000;
            XY[i][j][0] = -1000000;
            XZ[i][j][0] = -1000000;
            YZ[i][j][0] = -1000000;
            X[i][j][0] = -1000000;
            Y[i][j][0] = -1000000;
            Z[i][j][0] = -1000000;

            M[i][j][1] = -1000000;
            XY[i][j][1] = -1000000;
            XZ[i][j][1] = -1000000;
            YZ[i][j][1] = -1000000;
            X[i][j][1] = -1000000;
            Y[i][j][1] = -1000000;
            Z[i][j][1] = -1000000;
        }
    }
    for(int cell_idx=0; cell_idx < lookup_length; cell_idx++){
        
        if (lookup_table[cell_idx][2] != z_idx){
            z_idx++;
            for(int i=0; i<seq1_length+1; i++){
                for(int j=0; j<seq2_length+1; j++){

                    M[i][j][0]=M[i][j][1];
                    XY[i][j][0]=XY[i][j][1];
                    XZ[i][j][0]=XZ[i][j][1];
                    YZ[i][j][0]=YZ[i][j][1];
                    X[i][j][0]=X[i][j][1];
                    Y[i][j][0]=Y[i][j][1];
                    Z[i][j][0]=Z[i][j][1];

                    M[i][j][1] = -1000000;
                    XY[i][j][1] = -1000000;
                    XZ[i][j][1] = -1000000;
                    YZ[i][j][1] = -1000000;
                    X[i][j][1] = -1000000;
                    Y[i][j][1] = -1000000;
                    Z[i][j][1] = -1000000;
                }
            }
        }

        if(lookup_table[cell_idx][2] == z_idx){

            x = lookup_table[cell_idx][0];
            y = lookup_table[cell_idx][1];
            z = lookup_table[cell_idx][2];

            score12 = mmMatrix(s1[x-1],s2[y-1]);
            score13 = mmMatrix(s1[x-1],s3[z-1]);
            score23 = mmMatrix(s2[y-1],s3[z-1]);

            if(x-1<0 || y-1<0 || z-1<0){
                
                M[x][y][1] = -1000000;
                if(z==0)
                    M[0][0][1]  = 0;
            }
            else{
                score = score12 + score13+ score23;
                values[0] = M[x-1][y-1][0] + score;
                values[1] = XY[x-1][y-1][0] + score;
                values[2] = XZ[x-1][y-1][0] + score;
                values[3] = YZ[x-1][y-1][0] + score;
                values[4] = X[x-1][y-1][0] + score;
                values[5] = Y[x-1][y-1][0] + score;
                values[6] = Z[x-1][y-1][0] + score;

                M[x][y][1] = values[0];
               
                lookup_table[cell_idx][3] = 0;
                for (int ctr = 1; ctr < 7; ctr++ ){
                    if (M[x][y][1] < values[ctr]){
                        M[x][y][1] = values[ctr];
                        lookup_table[cell_idx][3] = ctr;
                    }
        
                }
                
            }
            if(x-1<0 || y-1<0){
                
                XY[x][y][1] = -1000000;
                if(z==0)
                    XY[0][0][1] = 0;
            }
            else{
                values[0] = M[x-1][y-1][1] - 2*GO + score12;
                values[2] = XZ[x-1][y-1][1] - 2*GO + score12;
                values[3] = YZ[x-1][y-1][1] - 2*GO + score12;
                values[6] = Z[x-1][y-1][1] - 2*GO + score12;

                if(z==0 && x==1 && y==1){
                    values[1] = XY[x-1][y-1][1] - 2*GO + score12;
                    values[4] = X[x-1][y-1][1] - 2*GO + score12;
                    values[5] = Y[x-1][y-1][1] - 2*GO + score12;
                }
                else{

                    values[1] = XY[x-1][y-1][1] - 2*GE + score12;
                    values[4] = X[x-1][y-1][1]  - 2*GE + score12;
                    values[5] = Y[x-1][y-1][1] - 2*GE + score12;

                }

                XY[x][y][1] = values[0];
           
                lookup_table[cell_idx][4] = 0;
                for (int ctr = 1; ctr < 7; ctr++ ){
                    if (XY[x][y][1] < values[ctr]){
                        XY[x][y][1] = values[ctr];
                        //BXY[x][y][z] = ctr;
                        lookup_table[cell_idx][4] = ctr;
                    }
                } 
            }
            if(x-1<0 || z-1<0){
                XZ[x][y][1] = -1000000;
                if(z==0)
                    XZ[0][0][1] = 0;
            }
            else{
                if(y==0 && x==1 && z == 1){

                    values[0] = M[x-1][y][0] - 2*GO + score13;
                    values[1] = XY[x-1][y][0] - 2*GO + score13;
                    values[2] = XZ[x-1][y][0] - 2*GO + score13;
                    values[3] = YZ[x-1][y][0] - 2*GO + score13;
                    values[4] = X[x-1][y][0] - 2*GO + score13;
                    values[5] = Y[x-1][y][0] - 2*GO + score13;
                    values[6] = Z[x-1][y][0] - 2*GO + score13;
                }
                else{

                    values[0] = M[x-1][y][0] - 2*GO + score13;
                    values[1] = XY[x-1][y][0] - 2*GO + score13;
                    values[2] = XZ[x-1][y][0] - 2*GE + score13;
                    values[3] = YZ[x-1][y][0] - 2*GO + score13;
                    values[4] = X[x-1][y][0] - 2*GE + score13;
                    values[5] = Y[x-1][y][0] - 2*GO + score13;
                    values[6] = Z[x-1][y][0] - 2*GE + score13;
                }
                XZ[x][y][1] = values[0];
                lookup_table[cell_idx][5] = 0;
                for (int ctr = 1; ctr < 7; ctr++ ){
                    if (XZ[x][y][1] < values[ctr]){
                        XZ[x][y][1] = values[ctr];
                        lookup_table[cell_idx][5] = ctr;
                    }
                }
            }

            if(y-1<0 || z-1<0){
                YZ[x][y][1] = -1000000;
                if(z==0)
                    YZ[0][0][1] = 0;   
            }
            else{

                if(x==0 && y==1 && z==1){
                    values[0] = M[x][y-1][0] - 2*GO + score23;
                    values[1] = XY[x][y-1][0] - 2*GO + score23;
                    values[2] = XZ[x][y-1][0] - 2*GO + score23;
                    values[3] = YZ[x][y-1][0] - 2*GO + score23;
                    values[4] = X[x][y-1][0] - 2*GO + score23;
                    values[5] = Y[x][y-1][0] - 2*GO + score23;
                    values[6] = Z[x][y-1][0] - 2*GO + score23;
                }
                else{
                    values[0] = M[x][y-1][0] - 2*GO + score23;
                    values[1] = XY[x][y-1][0] - 2*GO + score23;
                    values[2] = XZ[x][y-1][0] - 2*GO + score23;
                    values[3] = YZ[x][y-1][0] - 2*GE + score23;
                    values[4] = X[x][y-1][0] - 2*GO + score23;
                    values[5] = Y[x][y-1][0] - 2*GE + score23;
                    values[6] = Z[x][y-1][0] - 2*GE + score23;
                }

                YZ[x][y][1] = values[0];
                lookup_table[cell_idx][6] = 0;
                for (int ctr = 1; ctr < 7; ctr++ ){
                    if ( YZ[x][y][1] < values[ctr]){
                        YZ[x][y][1] = values[ctr];
                        lookup_table[cell_idx][6] = ctr;
                    }
        
                }
            }

            if(x-1<0){
                X[x][y][1] = -1000000;
                if(z==0)
                    X[0][0][1] = 0;
            }
            else{
                if(y==0 && z == 0 && x==1){ 
                    values[0] = M[x-1][y][1] - 2*GO;
                    values[1] = XY[x-1][y][1] - 2*GO;
                    values[2] = XZ[x-1][y][1] - 2*GO;
                    values[3] = YZ[x-1][y][1] - 2*GO;
                    values[4] = X[x-1][y][1] - 2*GO;
                    values[5] = Y[x-1][y][1] - 2*GO;
                    values[6] = Z[x-1][y][1] - 2*GO;
                        
                }
                else{
                    values[0] = M[x-1][y][1] - 2*GO;
                    values[1] = XY[x-1][y][1] - GO - GE;
                    values[2] = XZ[x-1][y][1] - GO - GE;
                    values[3] = YZ[x-1][y][1] - 2*GO;
                    values[4] = X[x-1][y][1] - 2*GE;
                    values[5] = Y[x-1][y][1] - GO - GE;
                    values[6] = Z[x-1][y][1] - GO - GE;
                }

                X[x][y][1] = values[0];
                lookup_table[cell_idx][7] = 0;
                for (int ctr = 1; ctr < 7; ctr++ ){
                    if (X[x][y][1] < values[ctr]){
                        X[x][y][1] = values[ctr];
                        lookup_table[cell_idx][7] = ctr;
                    }
                }
            }
            if(y-1<0){
                Y[x][y][1] = -1000000;
                if(z==0)
                    Y[0][0][1] = 0;
            }
            else{
                values[0] = M[x][y-1][1] - 2*GO;
                
                if(x == 0 && z == 0 && y==1){

                    values[1] = XY[x][y-1][1] - 2*GO;
                    values[2] = XZ[x][y-1][1] - 2*GO;
                    values[3] = YZ[x][y-1][1] - 2*GO;
                    values[4] = X[x][y-1][1] - 2*GO;
                    values[5] = Y[x][y-1][1] - 2*GO;
                    values[6] = Z[x][y-1][1] - 2*GO;
                }
                else{
                    values[1] = XY[x][y-1][1] - GO - GE;
                    values[2] = XZ[x][y-1][1] - 2*GO;
                    values[3] = YZ[x][y-1][1] - GO - GE;
                    values[4] = X[x][y-1][1] - GO - GE;
                    values[5] = Y[x][y-1][1] - 2*GE;
                    values[6] = Z[x][y-1][1] - GO - GE;
                }

                Y[x][y][1] = values[0];
                lookup_table[cell_idx][8] = 0;
                for (int ctr = 1; ctr < 7; ctr++ ){
                    if (Y[x][y][1] < values[ctr]){
                        Y[x][y][1] = values[ctr];
                        lookup_table[cell_idx][8] = ctr;
                    }
        
                }
            }
            if(z-1 < 0){
                Z[x][y][1] = -1000000;
                if(z==0)
                    Z[0][0][1] = 0;
            }
            else{
                values[0] = M[x][y][0] - 2*GO;
                values[1] = XY[x][y][0] - 2*GO;
                
                if(x == 0 && y == 0 && z==1) {

                    values[2] = XZ[x][y][0] - 2*GO;
                    values[3] = YZ[x][y][0] - 2*GO;
                    values[4] = X[x][y][0] - 2*GO;
                    values[5] = Y[x][y][0] - 2*GO;
                    values[6] = Z[x][y][0] - 2*GO;
                }
                else {
                    values[2] = XZ[x][y][0] - GO - GE;
                    values[3] = YZ[x][y][0] - GO - GE;
                    values[4] = X[x][y][0] - GO - GE;
                    values[5] = Y[x][y][0] - GO - GE;
                    values[6] = Z[x][y][0] - 2*GE;
                }

                Z[x][y][1] = values[0];
                lookup_table[cell_idx][9] = 0;
                for (int ctr = 1; ctr < 7; ctr++ ){
                    if (Z[x][y][1] < values[ctr]){
                        Z[x][y][1] = values[ctr];
                        lookup_table[cell_idx][9] = ctr;
                    }
                }
            }
        }
    }
    
    x = seq1_length;
    y = seq2_length;
    z = seq3_length;

    int val[7];
    val[0] = M[seq1_length][seq2_length][1];
    val[1] = XY[seq1_length][seq2_length][1];
    val[2] = XZ[seq1_length][seq2_length][1];
    val[3] = YZ[seq1_length][seq2_length][1];
    val[4] = X[seq1_length][seq2_length][1];
    val[5] = Y[seq1_length][seq2_length][1];
    val[6] = Z[seq1_length][seq2_length][1];

    max = val[0];
    idx = 0;
    for (int ctr = 1; ctr < 7; ctr++ ){
        if (max < val[ctr]){
            max = val[ctr];
            idx = ctr;
        }
    }
    int cell_idx = lookup_length - 1;
    switch (idx)
    {
    case 0 :
        BM_func(s1, s2, s3, cell_idx);
        break;
    case 1 :
        BXY_func(s1, s2, s3, cell_idx);
        break;
    case 2 :
        BXZ_func(s1, s2, s3, cell_idx);
        break;
    case 3 :
        BYZ_func(s1, s2, s3, cell_idx);
        break;
    case 4 :
        BX_func(s1, s2, s3, cell_idx);
        break;
    case 5 :
        BY_func(s1, s2, s3, cell_idx);
        break;
    case 6 :
        BZ_func(s1, s2, s3, cell_idx);
        break;
    default:
        break;
    }
}

void PrintDistanceMatrix(int num, double** matrix){
    FILE* fptr = fopen(mat_idx,"w");
    FILE* fptr2 = fopen(data_idx,"r");
    int rowCtr = 0;
    char buf[100];
    
    fprintf(fptr, "   %d\n", num);
    
    while(fgets(buf, 12, fptr2)!=NULL){
        if (buf[0] == '>'){
            
            fprintf(fptr, "%s", buf);
            fprintf(fptr, "     ");
            for(int i2 = 0; i2< num; i2++) {
                fprintf(fptr , "%.4f  " , matrix[rowCtr][i2]);
            }
            fprintf(fptr,"\n");
            rowCtr++;
        }
    }
    fclose(fptr);
    fclose(fptr2);
}

int Alignment_score_2D(int* seq_1, int* seq_2, int l){

    int score = 0;
    int gflag1 = 1;
    int gflag2 = 1;

    for(int i=0; i<l; i++){ 

        if(seq_1[i] != 0 && seq_2[i] != 0){
            score = score + mmMatrix(seq_1[i], seq_2[i]);
            gflag1 = 1;
            gflag2 = 1;
        }else if(seq_1[i] == 0 && seq_2[i] != 0){
            gflag2 = 1;
            if(gflag1 == 1){
                gflag1 = 0;
                score = score - GO;
            }
            else
                score = score - GE;
        } else if(seq_1[i] != 0 && seq_2[i] == 0){
            gflag1 = 1;
            if(gflag2 == 1){
                gflag2 = 0;
                score = score - GO;
            }
            else
                score = score - GE;
        }
    }
    return score;
}

int P_B0_func(int **s1, int *s2, int C, int R){
    
    if(R == 0 && C == 0 || R < 0 || C < 0)
        return 0;

    if (B0[R][C] == 0){ 
        if (R-1 < 0){
           
            lambda_e[0][p_ct1] = s1[0][C-1];
            lambda_e[1][p_ct1] = s1[1][C-1];
            lambda_e[2][p_ct2] = 0;
            p_ct1++;
            p_ct2++;
            e_length++;

        }
        else if (C-1 < 0){
          
            lambda_e[0][p_ct1] = 0;
            lambda_e[1][p_ct1] = 0;
            lambda_e[2][p_ct2] = s2[R-1];
            p_ct1++;
            p_ct2++;
            e_length++;
        }

        else{
            
            lambda_e[0][p_ct1] = s1[0][C-1];
            lambda_e[1][p_ct1] = s1[1][C-1];
            lambda_e[2][p_ct2] = s2[R-1];
            p_ct1++;
            p_ct2++;
            e_length++;
            
        }  
         
        P_B0_func(s1, s2, C-1, R-1);
    }
    else if(B0[R][C] == 1 || B0[R][C] == 3){

        P_B1_func(s1, s2, C, R);
    }
    else{
     
        P_B2_func(s1, s2, C, R);
    }
    return 0;

}

int P_B1_func(int **s1, int *s2, int C, int R){
    if(R == 0 && C == 0 || R < 0 || C < 0)
        return 0;

    if (B1[R][C] == 2){
        //printf("Hii\n");
        lambda_e[0][p_ct1] = s1[0][C-1];
        lambda_e[1][p_ct1] = s1[1][C-1];
        lambda_e[2][p_ct2] = 0;
        p_ct1++;
        p_ct2++;
        e_length++;
        P_B1_func(s1, s2, C-1, R); 
    }
    else{
        //printf("HIiiii\n");
        lambda_e[0][p_ct1] = s1[0][C-1];
        lambda_e[1][p_ct1] = s1[1][C-1];
        lambda_e[2][p_ct2] = 0;
        p_ct1++;
        p_ct2++;
        e_length++;
        P_B0_func(s1, s2, C-1, R);
    }
    return 0;
}

int P_B2_func(int **s1, int *s2, int C, int R){
    if(R == 0 && C == 0 || R < 0 || C < 0)
        return 0;
    if(B2[R][C] == 2){
        
        lambda_e[0][p_ct1] = 0;
        lambda_e[1][p_ct1] = 0;
        lambda_e[2][p_ct2] = s2[R-1];
        p_ct1++;
        p_ct2++;
        e_length++;
        P_B2_func(s1, s2, C, R-1);
    }
    else{
      
        lambda_e[0][p_ct1] = 0;
        lambda_e[1][p_ct1] = 0;
        lambda_e[2][p_ct2] = s2[R-1];
        p_ct1++;
        p_ct2++;
        e_length++;
        P_B0_func(s1, s2, C, R-1);
    }
    return 0;
}

void Profile_Alignment(int** s1, int* s2, int seq1_l, int seq2_l){

    lambda_e = malloc(sizeof(int*) * 3);
    for(int k=0; k<3; k++)
        lambda_e[k] = malloc(sizeof(int) * (max_size));
    
    M2 = malloc(sizeof(int*) * (seq2_l+1));
    X2 = malloc(sizeof(int*) * (seq2_l+1));
    Y2 = malloc(sizeof(int*) * (seq2_l+1));
    B0 = malloc(sizeof(int*) * (seq2_l+1));
    B1 = malloc(sizeof(int*) * (seq2_l+1));
    B2 = malloc(sizeof(int*) * (seq2_l+1));

    for (int i = 0; i < seq2_l+1; i++){
        M2[i] = malloc(sizeof(int) * (seq1_l + 1));
        X2[i] = malloc(sizeof(int) * (seq1_l + 1));
        Y2[i] = malloc(sizeof(int) * (seq1_l + 1));
        B0[i] = malloc(sizeof(int) * (seq1_l + 1));
        B1[i] = malloc(sizeof(int) * (seq1_l + 1));
        B2[i] = malloc(sizeof(int) * (seq1_l + 1));
    
    }

    for (int row = 1; row < seq2_l + 1; row++){
        Y2[row][0] = -2 * GO + (-2 * GE * (row - 1));
        X2[row][0] = 0;
        M2[row][0] = -2 * GO + (-2 * GE * (row - 1));
        B0[row][0] = 2;
        B2[row][0] = 2;
    }

    for (int col = 1; col < seq1_l + 1; col++){

        Y2[0][col] = 0;
        X2[0][col] = -2 * GO + (-2 * GE * (col - 1));
        M2[0][col] = -2 * GO + (-2 * GE * (col - 1));
        B0[0][col] = 1;
        B1[0][col] = 1;
    }

    int s, ss;
    int gap_e_1, gap_e_2;

    for(int row = 1; row < seq2_l + 1; row++){

        gap_e_1 = 0;
        gap_e_2 = 0;

        for(int col = 1; col < seq1_l + 1; col++){

            if(s1[0][col-1] != 0)
                gap_e_1 = 0;
            else
                gap_e_1++;
            if(s1[1][col-1] != 0)
                gap_e_2 = 0;
            else
                gap_e_2++;

            if(s1[0][col-1] != 0 && s1[1][col-1] != 0)
                s = mmMatrix(s1[0][col-1],s2[row-1]) + mmMatrix(s1[1][col-1],s2[row-1]);
            else if(s1[0][col-1] != 0 && s1[1][col-1] == 0){
                if(gap_e_2 > 1)
                    s = mmMatrix(s1[0][col-1],s2[row-1]) - GE;
                else
                    s = mmMatrix(s1[0][col-1],s2[row-1]) - GO;
            }
            else if(s1[0][col-1] == 0 && s1[1][col-1] != 0){
                if(gap_e_1 > 1)
                    s = mmMatrix(s1[1][col-1],s2[row-1]) - GE;
                else
                    s = mmMatrix(s1[1][col-1],s2[row-1]) - GO;
            }
            
            if(s1[0][col-1] != 0 && s1[1][col-1] != 0){
                X2[row][col] = max_function2(M2[row-1][col] - 2 * GO, X2[row-1][col] - 2 * GE);
                Y2[row][col] = max_function2(M2[row][col-1] - GO, Y2[row][col-1] -  GE);
                M2[row][col] = max_function2(max_function2(M2[row-1][col-1] + s, X2[row][col]), Y2[row][col]);

                if (M2[row][col] == M2[row-1][col-1] + s)
                    B0[row][col] = 0;
                else if (M2[row][col] == Y2[row][col])
                    B0[row][col] = 1;
                else 
                    B0[row][col] = 2;

                if(M2[row][col] == M2[row-1][col-1] + s && M2[row][col] == Y2[row][col])
                    B0[row][col] = 1;

                if(M2[row][col] == M2[row-1][col-1] + s && M2[row][col] == X2[row][col])
                    B0[row][col] = 2;

                if(M2[row][col] == X2[row][col] && M2[row][col] == Y2[row][col])
                    B0[row][col] = 3;
            
                if(Y2[row][col-1] - GE <= M2[row][col-1]-GO)
                    B1[row][col] = 1;
                else
                    B1[row][col] = 2;

                if(X2[row-1][col] - 2 * GE <= M2[row-1][col] -2 * GO)
                    B2[row][col] = 1;
                else
                    B2[row][col] = 2;
            }

            else if(s1[0][col-1] == 0 && s1[1][col-1] != 0 || s1[0][col-1] != 0 && s1[1][col-1] == 0){

                X2[row][col] = max_function2(M2[row-1][col] - GO - GE, X2[row-1][col] - 2 * GE);
                Y2[row][col] = max_function2(M2[row][col-1] - GO, Y2[row][col-1] - GE);
                M2[row][col] = max_function2(max_function2(M2[row-1][col-1] + s, X2[row][col]), Y2[row][col]);

                if (M2[row][col] == M2[row-1][col-1] + s)
                    B0[row][col] = 0;
                else if (M2[row][col] == Y2[row][col])
                    B0[row][col] = 1;
                else 
                    B0[row][col] = 2;
                if(M2[row][col] == M2[row-1][col-1] + s && M2[row][col] == Y2[row][col])
                    B0[row][col] = 1;
                if(M2[row][col] == M2[row-1][col-1] + s && M2[row][col] == X2[row][col])
                    B0[row][col] = 2;
                if(M2[row][col] == X2[row][col] && M2[row][col] == Y2[row][col])
                    B0[row][col] = 3;
                if(Y2[row][col-1] - GE <= M2[row][col-1]-GO)
                    B1[row][col] = 1;
                else
                    B1[row][col] = 2;
                if(X2[row-1][col] - 2 * GE <= M2[row-1][col] - GO - GE)
                    B2[row][col] = 1;
                else
                    B2[row][col] = 2;
            }
        }
    }
    int R = seq2_l;
    int C = seq1_l;
    P_B0_func(s1, s2, C, R);
}

int main(int argc, char *argv[]){

    if (argc != 4) {
        printf("Usage: %s GO GE num_seq scoring_mat\n", argv[0]);
        return 1; // Return an error code indicating incorrect usage
    }

    printf("Please enter Gap opening, Gap extension and number of sequences");

    GO = atoi(argv[1]);
    GE = atoi(argv[2]);
    num_seqs = atoi(argv[3]);


    int** proj12_candidates;
    int** proj13_candidates;
    int** proj23_candidates;
    int r;
    int a,b,c,a1,a2,s11,s22,s33;
    int Alignment_length12,  Alignment_length13,  Alignment_length23;
    int num_candidates_12, num_candidates_13, num_candidates_23; 
    int Alignment_length1, Alignment_length2;

    seq_lengths = malloc(sizeof(int) * num_seqs);
    ROSE_length = malloc(sizeof(int) * num_seqs);
    Seqs = malloc(sizeof(int*)* num_seqs);
    reverse_Seqs = malloc(sizeof(int*)* num_seqs);
    d_Matrix = malloc(sizeof(double*)* num_seqs);
    
    for (int i = 0; i < num_seqs; i++){
        Seqs[i] = malloc(sizeof(int) * (max_size));
        reverse_Seqs[i] = malloc(sizeof(int) * (max_size));
        d_Matrix[i] = malloc(sizeof(double) * (num_seqs));
    }
    
    M = malloc(sizeof(int**) * (max_size));
    XY = malloc(sizeof(int**) * (max_size));
    XZ = malloc(sizeof(int**) * (max_size));
    YZ = malloc(sizeof(int**) * (max_size));
    X = malloc(sizeof(int**) * (max_size));
    Y = malloc(sizeof(int**) * (max_size));
    Z = malloc(sizeof(int**) * (max_size));
    
    for(int i=0; i<max_size; i++){
        M[i] = malloc(sizeof(int*) * (max_size));
        XY[i] = malloc(sizeof(int*) * (max_size));
        XZ[i] = malloc(sizeof(int*) * (max_size));
        YZ[i] = malloc(sizeof(int*) * (max_size));
        X[i] = malloc(sizeof(int*) * (max_size));
        Y[i] = malloc(sizeof(int*) * (max_size));
        Z[i] = malloc(sizeof(int*) * (max_size));
        

        for(int j=0; j<max_size+1; j++){

            M[i][j] = malloc(sizeof(int) * (2));
            XY[i][j] = malloc(sizeof(int) * (2));
            XZ[i][j] = malloc(sizeof(int) * (2));
            YZ[i][j] = malloc(sizeof(int) * (2));
            X[i][j] = malloc(sizeof(int) * (2));
            Y[i][j] = malloc(sizeof(int) * (2));
            Z[i][j] = malloc(sizeof(int) * (2));
            
        }
    }

    AlignedPair = malloc(sizeof(int*) * 2);
    Alignment12 = malloc(sizeof(int*) * 2);
    Alignment13 = malloc(sizeof(int*) * 2);
    Alignment23 = malloc(sizeof(int*) * 2);
    Alignment1 = malloc(sizeof(int*) * 2);
    Alignment2 = malloc(sizeof(int*) * 2);

    for (int k=0; k<2; k++){
        AlignedPair[k] = malloc(sizeof(int) * (max_size));
        Alignment12[k] = malloc(sizeof(int) * (max_size));
        Alignment13[k] = malloc(sizeof(int) * (max_size));
        Alignment23[k] = malloc(sizeof(int) * (max_size));
        Alignment1[k] = malloc(sizeof(int) * (max_size));
        Alignment2[k] = malloc(sizeof(int) * (max_size));
    }

    score_matrix_12 = malloc(sizeof(int*) * max_size);
    score_matrix_13 = malloc(sizeof(int*) * max_size);
    score_matrix_23 = malloc(sizeof(int*) * max_size);

    fMM = malloc(sizeof(int*) * max_size);
    fX2 = malloc(sizeof(int*) * max_size);
    fY2 = malloc(sizeof(int*) * max_size);

    for(int k=0; k<max_size; k++){

        score_matrix_12[k] = malloc(sizeof(int) * max_size);
        score_matrix_13[k] = malloc(sizeof(int) * max_size);
        score_matrix_23[k] = malloc(sizeof(int) * max_size);

        fMM[k] = malloc(sizeof(int) * max_size);
        fX2[k] = malloc(sizeof(int) * max_size);
        fY2[k] = malloc(sizeof(int) * max_size);

    }

    proj12_candidates = malloc(sizeof(int*) * (max_size));
    proj13_candidates = malloc(sizeof(int*) * (max_size));
    proj23_candidates = malloc(sizeof(int*) * (max_size));

    for(int i=0; i< max_size; i++){

        proj12_candidates[i] = malloc(sizeof(int) * max_size);
        proj13_candidates[i] = malloc(sizeof(int) * max_size);
        proj23_candidates[i] = malloc(sizeof(int) * max_size);

    }

    int **cell_candidates;

    AlignedTriplet = malloc(sizeof(int*) * 3);
    AlignedTriplet2 = malloc(sizeof(int*) * 3);
    for (int k=0; k<3; k++){
        AlignedTriplet[k] = malloc(sizeof(int) * (2000));
        AlignedTriplet2[k] = malloc(sizeof(int) * (2000));
    }


    strcpy(mat_idx, "distmatrix");
    strcpy(gaplessidx, "gapless");
    strcpy(data_idx, "data");
    strcat(mat_idx, ".txt");
    strcat(data_idx, ".fa");

    Readsequences(); // Reading iqtree file 

    for(int i=0; i<num_seqs; i++){
        for(int j=0; j<seq_lengths[i]; j++){
            reverse_Seqs[i][j] = Seqs[i][seq_lengths[i] - j -1];
        }
    }

    for(int i=0; i<num_seqs; i++){
        for (int j=0; j<num_seqs; j++){
            d_Matrix[i][j] = 0;
        }
            
    }

    for(int seq1 = 0; seq1< num_seqs-2; seq1++){  // loop over all triplets
        for(int seq2 = seq1+1; seq2< num_seqs-1 ; seq2++){
            for(int seq3 = seq2+1; seq3 < num_seqs; seq3++){

                seq1_length = seq_lengths[seq1];
                seq2_length = seq_lengths[seq2];
                seq3_length = seq_lengths[seq3];

                alignment_length = 0;
                ct1 = 0;
                ct2 = 0;

                Alignment_2D(Seqs[seq1], Seqs[seq2], seq1_length, seq2_length); 

                Alignment_length12 = alignment_length;

                for(int i=0; i<alignment_length; i++){
                    Alignment12[0][i] = AlignedPair[0][i];
                    Alignment12[1][i] = AlignedPair[1][i];

                }
                for(int i=0; i<seq2_length+1; i++){
                    for(int j=0; j<seq1_length+1; j++){
                        
                        fMM[i][j] = MM[i][j];
                        fX2[i][j] = X2[i][j];
                        fY2[i][j] = Y2[i][j];
                    }
                }
                alignment_length = 0;
                ct1 = 0;
                ct2 = 0;
                
                Alignment_2D(reverse_Seqs[seq1], reverse_Seqs[seq2], seq1_length, seq2_length); // reverse 1 and 2

                for(int i=0; i<seq2_length+1; i++){
                    for(int j=0; j<seq1_length+1; j++){
                        
                        a = max_function2(max_function2(fMM[i][j]+MM[seq2_length - i][seq1_length - j], fY2[i][j] + Y2[seq2_length - i][seq1_length - j]+GO-GE), fX2[i][j] + X2[seq2_length - i][seq1_length - j]+GO-GE);
                        b = max_function2(max_function2(fMM[i][j]+X2[seq2_length - i][seq1_length - j], fMM[i][j] + Y2[seq2_length - i][seq1_length - j]), fX2[i][j] + Y2[seq2_length - i][seq1_length - j]);
                        c = max_function2(max_function2(fX2[i][j]+MM[seq2_length - i][seq1_length - j], fY2[i][j] + MM[seq2_length - i][seq1_length - j]), fY2[i][j] + X2[seq2_length - i][seq1_length - j]);
                        score_matrix_12[i][j] = max_function2(max_function2(a,b),c);
                    }
                }
                score_matrix_12[seq2_length][seq1_length] = max_function2(max_function2(fMM[seq2_length][seq1_length], 
                fY2[seq2_length][seq1_length]), fX2[seq2_length][seq1_length]);
                score_matrix_12[0][0] = max_function2(max_function2(fMM[seq2_length][seq1_length], 
                fY2[seq2_length][seq1_length]), fX2[seq2_length][seq1_length]);


                alignment_length = 0;
                ct1 = 0;
                ct2 = 0;


                Alignment_2D(Seqs[seq1], Seqs[seq3], seq1_length, seq3_length);

                Alignment_length13 = alignment_length;
                for(int i=0; i<alignment_length; i++){
                    Alignment13[0][i] = AlignedPair[0][i];
                    Alignment13[1][i] = AlignedPair[1][i];

                }
                
                for(int i=0; i<seq3_length+1; i++){
                    for(int j=0; j<seq1_length+1; j++){
                        
                        fMM[i][j] = MM[i][j];
                        fX2[i][j] = X2[i][j];
                        fY2[i][j] = Y2[i][j];
                    }
                }
                
                alignment_length = 0;
                ct1 = 0;
                ct2 = 0;

                Alignment_2D(reverse_Seqs[seq1], reverse_Seqs[seq3], seq1_length, seq3_length);
                

                for(int i=0; i<seq3_length+1; i++){
                    for(int j=0; j<seq1_length+1; j++){
                        a = max_function2(max_function2(fMM[i][j]+MM[seq3_length - i][seq1_length - j], fY2[i][j] + Y2[seq3_length - i][seq1_length - j]+GO-GE), fX2[i][j] + X2[seq3_length - i][seq1_length - j]+GO-GE);
                        b = max_function2(max_function2(fMM[i][j]+X2[seq3_length - i][seq1_length - j], fMM[i][j] + Y2[seq3_length - i][seq1_length - j]), fX2[i][j] + Y2[seq3_length - i][seq1_length - j]);
                        c = max_function2(max_function2(fX2[i][j]+MM[seq3_length - i][seq1_length - j], fY2[i][j] + MM[seq3_length - i][seq1_length - j]), fY2[i][j] + X2[seq3_length - i][seq1_length - j]);
                        score_matrix_13[i][j] = max_function2(max_function2(a,b),c);
                    }
                }

                score_matrix_13[seq3_length][seq1_length] = max_function2(max_function2(fMM[seq3_length][seq1_length], 
                fY2[seq3_length][seq1_length]), fX2[seq3_length][seq1_length]);
                score_matrix_13[0][0] = max_function2(max_function2(fMM[seq3_length][seq1_length], 
                fY2[seq3_length][seq1_length]), fX2[seq3_length][seq1_length]);
            
                alignment_length = 0;
                ct1 = 0;
                ct2 = 0;

                Alignment_2D(Seqs[seq2], Seqs[seq3], seq2_length, seq3_length);
            
                Alignment_length23 = alignment_length;

                for(int i=0; i<alignment_length; i++){
                    Alignment23[0][i] = AlignedPair[0][i];
                    Alignment23[1][i] = AlignedPair[1][i];

                }

                for(int i=0; i<seq3_length+1; i++){
                    for(int j=0; j<seq2_length+1; j++){
                        
                        fMM[i][j] = MM[i][j];
                        fX2[i][j] = X2[i][j];
                        fY2[i][j] = Y2[i][j];
                    }
                }
            
                alignment_length = 0;
                ct1 = 0;
                ct2 = 0;

                Alignment_2D(reverse_Seqs[seq2], reverse_Seqs[seq3], seq2_length, seq3_length); // reverse 2 and 3
                
                for(int i=0; i<seq3_length+1; i++){
                    for(int j=0; j<seq2_length+1; j++){
                        
                        a = max_function2(max_function2(fMM[i][j]+MM[seq3_length - i][seq2_length - j], fY2[i][j] + Y2[seq3_length - i][seq2_length - j]+GO-GE), fX2[i][j] + X2[seq3_length - i][seq2_length - j]+GO-GE);
                        b = max_function2(max_function2(fMM[i][j]+X2[seq3_length - i][seq2_length - j], fMM[i][j] + Y2[seq3_length - i][seq2_length - j]), fX2[i][j] + Y2[seq3_length - i][seq2_length - j]);
                        c = max_function2(max_function2(fX2[i][j]+MM[seq3_length - i][seq2_length - j], fY2[i][j] + MM[seq3_length - i][seq2_length - j]), fY2[i][j] + X2[seq3_length - i][seq2_length - j]);
                        score_matrix_23[i][j] = max_function2(max_function2(a,b),c);
                    }
                }
                score_matrix_23[seq3_length][seq2_length] = max_function2(max_function2(fMM[seq3_length][seq2_length], 
                fY2[seq3_length][seq2_length]), fX2[seq3_length][seq2_length]);
                score_matrix_23[0][0] = max_function2(max_function2(fMM[seq3_length][seq2_length], 
                fY2[seq3_length][seq2_length]), fX2[seq3_length][seq2_length]);

                for(int i=0; i< Alignment_length12/2; i++){
                    a1 = Alignment12[0][i];
                    a2 = Alignment12[1][i];

                    Alignment12[0][i] = Alignment12[0][Alignment_length12 - i - 1];
                    Alignment12[1][i] = Alignment12[1][Alignment_length12 - i - 1];

                    Alignment12[0][Alignment_length12 - i - 1] = a1;
                    Alignment12[1][Alignment_length12 - i - 1] = a2;

                }

                for(int i=0; i< Alignment_length13/2 ; i++){
                    a1 = Alignment13[0][i];
                    a2 = Alignment13[1][i];

                    Alignment13[0][i] = Alignment13[0][Alignment_length13 - i - 1];
                    Alignment13[1][i] = Alignment13[1][Alignment_length13 - i  -1];

                    Alignment13[0][Alignment_length13 - i - 1] = a1;
                    Alignment13[1][Alignment_length13 - i - 1] = a2;

                }
                for(int i=0; i< Alignment_length23/2; i++){
                    a1 = Alignment23[0][i];
                    a2 = Alignment23[1][i];

                    Alignment23[0][i] = Alignment23[0][Alignment_length23 - i - 1];
                    Alignment23[1][i] = Alignment23[1][Alignment_length23 - i - 1];

                    Alignment23[0][Alignment_length23 - i - 1] = a1;
                    Alignment23[1][Alignment_length23 - i - 1] = a2;
                }
                
                score12 = Alignment_score_2D(Alignment12[0], Alignment12[1], Alignment_length12);
                score13 = Alignment_score_2D(Alignment13[0], Alignment13[1],Alignment_length13);
                score23 = Alignment_score_2D(Alignment23[0], Alignment23[1],Alignment_length23);

                p_ct1 = 0;
                p_ct2 = 0;
                e_length = 0;
            
                if(score12>= score13 && score12 >= score23){ 

                    Profile_Alignment(Alignment12, Seqs[seq3], Alignment_length12, seq3_length);
                    s11 = Alignment_score_2D(lambda_e[0], lambda_e[1], e_length);
                    s22 = Alignment_score_2D(lambda_e[0], lambda_e[2], e_length);
                    s33 = Alignment_score_2D(lambda_e[1], lambda_e[2], e_length);
                }
                else if(score13>= score12 && score13>=score23){

                    Profile_Alignment(Alignment13, Seqs[seq2], Alignment_length13, seq2_length);
                    s11 = Alignment_score_2D(lambda_e[0], lambda_e[2], e_length);
                    s22 = Alignment_score_2D(lambda_e[0], lambda_e[1], e_length);
                    s33 = Alignment_score_2D(lambda_e[2], lambda_e[1], e_length);
                }
                else if(score23>= score13 && score23>=score12){

                    Profile_Alignment(Alignment23, Seqs[seq1], Alignment_length23, seq1_length);
                    s11 = Alignment_score_2D(lambda_e[1], lambda_e[2], e_length);
                    s22 = Alignment_score_2D(lambda_e[1], lambda_e[0], e_length);
                    s33 = Alignment_score_2D(lambda_e[2], lambda_e[0], e_length);
                }


                score_e = s11+s22+s33;
                int score_org = score_e;
                
                L12 = score_e - (score13+score23); // bounds defined by Carrillo-Lipman
                L13 = score_e - (score12+score23);
                L23 = score_e - (score12+score13);

                r = 0;
                num_candidates_12 = 0;

                for(int i=0; i<seq1_length+1; i++){
                    r = 0;
                    for(int j=0; j<seq2_length+1; j++){
                        if(score_matrix_12[j][i] >= L12){

                            num_candidates_12++;
                            proj12_candidates[i][r] = j;//seq1
                        
                            r++;
                            
                        }
                        else{
                            proj12_candidates[i][r] = -1;
                            r++;
                        }
                    }
                }

                num_candidates_13 = 0;
                for(int i=0; i<seq1_length+1; i++){

                    r = 0;
                    for(int j=0; j<seq3_length+1; j++){

                        if(score_matrix_13[j][i] >= L13){
                            num_candidates_13++;
                            proj13_candidates[i][r] = j;
                            r++;   
                        }
                        else{
                            proj13_candidates[i][r] = -1;
                            r++;
                        }
                    }
                }
                num_candidates_23 = 0;
                
                for(int i=0; i<seq2_length+1; i++){
                    for(int j=0; j<seq3_length+1; j++){
                        
                        if(score_matrix_23[j][i] >= L23){
                            num_candidates_23++;
                            proj23_candidates[i][j] = 1;
                        }
                        else
                            proj23_candidates[i][j] = 0;
                    }
                }
                r = 0;
                for(int r1 =0; r1<seq1_length + 1; r1++){
                    for(int c1=0; c1<seq2_length + 1; c1++){
                        if(proj12_candidates[r1][c1] != -1){
                            for(int k=0; k<seq3_length + 1; k++){ 
                                if(proj13_candidates[r1][k] != -1){ 
                                    
                                    if(proj23_candidates[proj12_candidates[r1][c1]][proj13_candidates[r1][k]] == 1){
                                        r++;
                                    }
                                }
                            }
                        }
                    }
                }
                num_cell_candidates = r;
                int p_num_can = r;
                lookup_length = r;
                ctr1 = 0;
                r=0;
                for(int r1 =0; r1<seq1_length + 1; r1++){
                    for(int c1=0; c1<seq2_length + 1; c1++){
                        
                        if(proj12_candidates[r1][c1] != -1){
                            for(int k=0; k<seq3_length + 1; k++){ 
                                
                                if(proj13_candidates[r1][k] != -1){ 
                                    
                                    if(proj23_candidates[proj12_candidates[r1][c1]][proj13_candidates[r1][k]] == 1){
                                        
                                        lookup_table[r][0] = proj12_candidates[r1][c1];//seq2_idx
                                        lookup_table[r][1] = proj13_candidates[r1][k];//seq3_idx
                                        lookup_table[r][2] = r1;//seq1_idx
                                        r++;
                                    }
                                }
                            }
                        }
                    }
                }
                    
                alignment_length = 0;
                seq1_length = seq_lengths[seq2];
                seq2_length = seq_lengths[seq3];
                seq3_length = seq_lengths[seq1];
                dynamic_prog(Seqs[seq2], Seqs[seq3], Seqs[seq1]);
                alignment_length = ctr1;
                
                for(int l=0; l<alignment_length; l++){

                    AlignedTriplet2[0][l] = AlignedTriplet[2][alignment_length - 1 - l];
                    AlignedTriplet2[1][l] = AlignedTriplet[0][alignment_length - 1 - l];
                    AlignedTriplet2[2][l] = AlignedTriplet[1][alignment_length - 1 - l];
                }
                Distance_matrix(AlignedTriplet2[0],AlignedTriplet2[1], alignment_length, seq1, seq2);
                Distance_matrix(AlignedTriplet2[0],AlignedTriplet2[2], alignment_length, seq1, seq3);
                Distance_matrix(AlignedTriplet2[1],AlignedTriplet2[2], alignment_length, seq2, seq3);
            }
        }
    }
    
    for(int i=0; i<num_seqs; i++){ 
        for (int j=0; j<num_seqs; j++)
            d_Matrix[i][j] = d_Matrix[i][j]/(num_seqs - 2);
    }
    PrintDistanceMatrix(num_seqs, d_Matrix);
}


static double jtteigs[] =
{+0.00000000000000,-1.81721720738768,-1.87965834528616,-1.61403121885431,
-1.53896608443751,-1.40486966367848,-1.30995061286931,-1.24668414819041,
-1.17179756521289,-0.31033320987464,-0.34602837857034,-1.06031718484613,
-0.99900602987105,-0.45576774888948,-0.86014403434677,-0.54569432735296,
-0.76866956571861,-0.60593589295327,-0.65119724379348,-0.70249806480753};

static double jttprobs[20][20] =
{{+0.07686196156903,+0.05105697447152,+0.04254597872702,+0.05126897436552,
 +0.02027898986051,+0.04106097946952,+0.06181996909002,+0.07471396264303,
 +0.02298298850851,+0.05256897371552,+0.09111095444453,+0.05949797025102,
 +0.02341398829301,+0.04052997973502,+0.05053197473402,+0.06822496588753,
 +0.05851797074102,+0.01433599283201,+0.03230298384851,+0.06637396681302},
{-0.04445795120462,-0.01557336502860,-0.09314817363516,+0.04411372100382,
 -0.00511178725134,+0.00188472427522,-0.02176250428454,-0.01330231089224,
 +0.01004072641973,+0.02707838224285,-0.00785039050721,+0.02238829876349,
 +0.00257470703483,-0.00510311699563,-0.01727154263346,+0.20074235330882,
 -0.07236268502973,-0.00012690116016,-0.00215974664431,-0.01059243778174},
{+0.09480046389131,+0.00082658405814,+0.01530023104155,-0.00639909042723,
 +0.00160605602061,+0.00035896642912,+0.00199161318384,-0.00220482855717,
 -0.00112601328033,+0.14840201765438,-0.00344295714983,-0.00123976286718,
 -0.00439399942758,+0.00032478785709,-0.00104270266394,-0.02596605592109,
 -0.05645800566901,+0.00022319903170,-0.00022792271829,-0.16133258048606},
{-0.06924141195400,-0.01816245289173,-0.08104005811201,+0.08985697111009,
 +0.00279659017898,+0.01083740322821,-0.06449599336038,+0.01794514261221,
 +0.01036809141699,+0.04283504450449,+0.00634472273784,+0.02339134834111,
 -0.01748667848380,+0.00161859106290,+0.00622486432503,-0.05854130195643,
 +0.15083728660504,+0.00030733757661,-0.00143739522173,-0.05295810171941},
{-0.14637948915627,+0.02029296323583,+0.02615316895036,-0.10311538564943,
 -0.00183412744544,-0.02589124656591,+0.11073673851935,+0.00848581728407,
 +0.00106057791901,+0.05530240732939,-0.00031533506946,-0.03124002869407,
 -0.01533984125301,-0.00288717337278,+0.00272787410643,+0.06300929916280,
 +0.07920438311152,-0.00041335282410,-0.00011648873397,-0.03944076085434},
{-0.05558229086909,+0.08935293782491,+0.04869509588770,+0.04856877988810,
 -0.00253836047720,+0.07651693957635,-0.06342453535092,-0.00777376246014,
 -0.08570270266807,+0.01943016473512,-0.00599516526932,-0.09157595008575,
 -0.00397735155663,-0.00440093863690,-0.00232998056918,+0.02979967701162,
 -0.00477299485901,-0.00144011795333,+0.01795114942404,-0.00080059359232},
{+0.05807741644682,+0.14654292420341,-0.06724975334073,+0.02159062346633,
 -0.00339085518294,-0.06829036785575,+0.03520631903157,-0.02766062718318,
 +0.03485632707432,-0.02436836692465,-0.00397566003573,-0.10095488644404,
 +0.02456887654357,+0.00381764117077,-0.00906261340247,-0.01043058066362,
 +0.01651199513994,-0.00210417220821,-0.00872508520963,-0.01495915462580},
{+0.02564617106907,+0.02960554611436,-0.00052356748770,+0.00989267817318,
 -0.00044034172141,-0.02279910634723,-0.00363768356471,-0.01086345665971,
 +0.01229721799572,+0.02633650142592,+0.06282966783922,-0.00734486499924,
 -0.13863936313277,-0.00993891943390,-0.00655309682350,-0.00245191788287,
 -0.02431633805559,-0.00068554031525,-0.00121383858869,+0.06280025239509},
{+0.11362428251792,-0.02080375718488,-0.08802750967213,-0.06531316372189,
 -0.00166626058292,+0.06846081717224,+0.07007301248407,-0.01713112936632,
 -0.05900588794853,-0.04497159138485,+0.04222484636983,+0.00129043178508,
 -0.01550337251561,-0.01553102163852,-0.04363429852047,+0.01600063777880,
 +0.05787328925647,-0.00008265841118,+0.02870014572813,-0.02657681214523},
{+0.01840541226842,+0.00610159018805,+0.01368080422265,+0.02383751807012,
 -0.00923516894192,+0.01209943150832,+0.02906782189141,+0.01992384905334,
 +0.00197323568330,+0.00017531415423,-0.01796698381949,+0.01887083962858,
 -0.00063335886734,-0.02365277334702,+0.01209445088200,+0.01308086447947,
 +0.01286727242301,-0.11420358975688,-0.01886991700613,+0.00238338728588},
{-0.01100105031759,-0.04250695864938,-0.02554356700969,-0.05473632078607,
 +0.00725906469946,-0.03003724918191,-0.07051526125013,-0.06939439879112,
 -0.00285883056088,+0.05334304124753,+0.12839241846919,-0.05883473754222,
 +0.02424304967487,+0.09134510778469,-0.00226003347193,-0.01280041778462,
 -0.00207988305627,-0.02957493909199,+0.05290385686789,+0.05465710875015},
{-0.01421274522011,+0.02074863337778,-0.01006411985628,+0.03319995456446,
 -0.00005371699269,-0.12266046460835,+0.02419847062899,-0.00441168706583,
 -0.08299118738167,-0.00323230913482,+0.02954035119881,+0.09212856795583,
 +0.00718635627257,-0.02706936115539,+0.04473173279913,-0.01274357634785,
 -0.01395862740618,-0.00071538848681,+0.04767640012830,-0.00729728326990},
{-0.03797680968123,+0.01280286509478,-0.08614616553187,-0.01781049963160,
 +0.00674319990083,+0.04208667754694,+0.05991325707583,+0.03581015660092,
 -0.01529816709967,+0.06885987924922,-0.11719120476535,-0.00014333663810,
 +0.00074336784254,+0.02893416406249,+0.07466151360134,-0.08182016471377,
 -0.06581536577662,-0.00018195976501,+0.00167443595008,+0.09015415667825},
{+0.03577726799591,-0.02139253448219,-0.01137813538175,-0.01954939202830,
 -0.04028242801611,-0.01777500032351,-0.02106862264440,+0.00465199658293,
 -0.02824805812709,+0.06618860061778,+0.08437791757537,-0.02533125946051,
 +0.02806344654855,-0.06970805797879,+0.02328376968627,+0.00692992333282,
 +0.02751392122018,+0.01148722812804,-0.11130404325078,+0.07776346000559},
{-0.06014297925310,-0.00711674355952,-0.02424493472566,+0.00032464353156,
 +0.00321221847573,+0.03257969053884,+0.01072805771161,+0.06892027923996,
 +0.03326534127710,-0.01558838623875,+0.13794237677194,-0.04292623056646,
 +0.01375763233229,-0.11125153774789,+0.03510076081639,-0.04531670712549,
 -0.06170413486351,-0.00182023682123,+0.05979891871679,-0.02551802851059},
{-0.03515069991501,+0.02310847227710,+0.00474493548551,+0.02787717003457,
 -0.12038329679812,+0.03178473522077,+0.04445111601130,-0.05334957493090,
 +0.01290386678474,-0.00376064171612,+0.03996642737967,+0.04777677295520,
 +0.00233689200639,+0.03917715404594,-0.01755598277531,-0.03389088626433,
 -0.02180780263389,+0.00473402043911,+0.01964539477020,-0.01260807237680},
{-0.04120428254254,+0.00062717164978,-0.01688703578637,+0.01685776910152,
 +0.02102702093943,+0.01295781834163,+0.03541815979495,+0.03968150445315,
 -0.02073122710938,-0.06932247350110,+0.11696314241296,-0.00322523765776,
 -0.01280515661402,+0.08717664266126,+0.06297225078802,-0.01290501780488,
 -0.04693925076877,-0.00177653675449,-0.08407812137852,-0.08380714022487},
{+0.03138655228534,-0.09052573757196,+0.00874202219428,+0.06060593729292,
 -0.03426076652151,-0.04832468257386,+0.04735628794421,+0.14504653737383,
 -0.01709111334001,-0.00278794215381,-0.03513813820550,-0.11690294831883,
 -0.00836264902624,+0.03270980973180,-0.02587764129811,+0.01638786059073,
 +0.00485499822497,+0.00305477087025,+0.02295754527195,+0.00616929722958},
{-0.04898722042023,-0.01460879656586,+0.00508708857036,+0.07730497806331,
 +0.04252420017435,+0.00484232580349,+0.09861807969412,-0.05169447907187,
 -0.00917820907880,+0.03679081047330,+0.04998537112655,+0.00769330211980,
 +0.01805447683564,-0.00498723245027,-0.14148416183376,-0.05170281760262,
 -0.03230723310784,-0.00032890672639,-0.02363523071957,+0.03801365471627},
{-0.02047562162108,+0.06933781779590,-0.02101117884731,-0.06841945874842,
 -0.00860967572716,-0.00886650271590,-0.07185241332269,+0.16703684361030,
 -0.00635847581692,+0.00811478913823,+0.01847205842216,+0.06700967948643,
 +0.00596607376199,+0.02318239240593,-0.10552958537847,-0.01980199747773,
 -0.02003785382406,-0.00593392430159,-0.00965391033612,+0.00743094349652}};

void PairDistance(int a1, int a2){
    double elambdat, q;
    
    for (int m =0; m < 20; m++){

        elambdat = exp(D * jtteigs[m]);

        q = jttprobs[m][a1] * jttprobs[m][a2] * elambdat;
        p += q;
        dp += jtteigs[m] * q;

        d2p += jtteigs[m] * jtteigs[m] * q;
    }
}
int BLOSUM62[20][20] = {
   {4, -1, -2,-2,  0 ,-1 ,-1 ,0, -2, -1, -1, -1, -1, -2, -1,  1,  0, -3, -2,  0}, 
  {-1,  5,  0,-2, -3  ,1  ,0, -2,  0, -3, -2,  2, -1, -3, -2, -1, -1, -3, -2, -3},  
  {-2, 0 , 6 ,1, -3  ,0  ,0 , 0,  1, -3, -3,  0, -2, -3, -2,  1,  0, -4, -2, -3},
  {-2, -2, 1 ,6, -3  ,0 , 2 ,-1, -1, -3, -4, -1, -3, -3, -1,  0, -1, -4, -3, -3}, 
   {0, -3,-3 ,-3,  9 ,-3 ,-4 ,-3, -3, -1, -1, -3, -1, -2, -3, -1, -1, -2, -2, -1},
  {-1, 1 , 0 , 0, -3 , 5 , 2 ,-2,  0, -3, -2,  1,  0, -3, -1,  0, -1, -2, -1, -2},
  {-1,0  ,0  , 2 ,-4 , 2 , 5 ,-2,  0, -3, -3,  1, -2, -3, -1,  0, -1, -3, -2, -2},
  { 0,-2 , 0 ,-1 ,-3 ,-2 ,-2 , 6, -2, -4, -4, -2, -3, -3, -2,  0, -2, -2, -3, -3},
  {-2, 0 , 1 ,-1 ,-3 , 0 , 0 ,-2,  8, -3, -3, -1, -2, -1, -2, -1, -2, -2,  2, -3},
  {-1,-3 ,-3 ,-3 ,-1 ,-3 ,-3 ,-4, -3,  4,  2, -3,  1,  0, -3, -2, -1, -3, -1,  3},
  {-1,-2 ,-3 ,-4 ,-1 ,-2 ,-3 ,-4, -3,  2,  4, -2,  2,  0, -3, -2 ,-1 ,-2, -1,  1},
  {-1, 2 , 0 ,-1 ,-3 , 1 , 1 ,-2, -1, -3, -2,  5, -1, -3, -1,  0, -1, -3, -2, -2},
  {-1,-1 ,-2 ,-3 ,-1 , 0 ,-2 ,-3, -2 , 1,  2, -1,  5,  0, -2, -1, -1, -1, -1,  1},
  {-2,-3 ,-3 ,-3 ,-2 ,-3 ,-3 ,-3,-1 , 0,  0, -3,  0,  6, -4, -2, -2,  1,  3, -1},
  {-1,-2 ,-2 ,-1 ,-3 ,-1 ,-1 ,-2, -2, -3, -3, -1, -2, -4,  7, -1, -1, -4, -3, -2},
   {1,-1 , 1 , 0 ,-1 , 0  ,0  ,0, -1, -2, -2,  0, -1, -2, -1,  4,  1, -3, -2, -2}, 
   {0,-1 , 0 ,-1 ,-1 ,-1 ,-1 ,-2, -2, -1, -1, -1, -1, -2, -1,  1,  5, -2, -2,  0},
  {-3, -3, -4 ,-4 ,-2 ,-2 ,-3, -2, -2, -3, -2, -3, -1,  1, -4, -3, -2, 11,  2, -3},
  {-2, -2, -2 ,-3 ,-2 ,-1 ,-2 ,-3,  2, -1, -1, -2, -1,  3, -3, -2, -2,  2,  7, -1},
  { 0, -3, -3 , -3 ,-1, -2 ,-2 ,-3, -3,  3,  1, -2,  1, -1, -2, -2,  0 ,-3, -1,  4}
};

int mmMatrix(int nn1, int nn2){

    int   Amino_num[25] = {1,0,2,3,4,5,6,7,8,0,9,10,11,12,0,13,14,15,16,17,0,18,19,0,20};

    char AA[] = {'A',  'R',  'N',  'D',  'C',  'Q',  'E', 'G',  'H',  'I',  'L',  'K',  'M', 'F',  'P', 'S', 'T',  'W',  'Y', 'V', '\0'};

    char c1, c2;
    int n1,n2;
    
    int score;
    for (int i=0; i<25; i++){
        if (nn1 == Amino_num[i])
            n1 = i+1;
        if (nn2 == Amino_num[i])
            n2 = i+1;
    }
    c1 = (char)(n1+64);
    c2 = (char)(n2+64);

    for (int i=0; i < 20; i++){
        if (AA[i] == c1){
            for(int j=0; j<20; j++){
                if(AA[j] == c2){
                    if(mmflag == 0)
                        score = BLOSUM62[i][j];  
                    //else if (mmflag == 1)
                        //score = BLOSUM30[i][j];
                    //else
                        //score = BLOSUM90[i][j];
                }
            }
        }
    }

    return score;
}



